import { defineComponent, ref } from "vue";
import PureTable from "@pureadmin/table";
import { pageContentProps } from "./types";
import { TableProBar } from "../../ReTable";
import { createActionProps } from "./utils/create-operation";
import { useRequeset } from "./hooks/use-request";
import { useSelection } from "./hooks/use-selection";
import { useRenderIcon } from "../../ReIcon/src/hooks";
import { useColumns } from "./hooks/use-columns";
import { useActions } from "./hooks/use-actions";
import { onBeforeRouteLeave } from "vue-router";
import { Operation } from "./actions";
import "./styles/index.scss";

import {
  ElButton,
  ElButtonGroup,
  ElCheckbox,
  ElCheckboxGroup,
  ElScrollbar,
  ElTooltip
} from "element-plus";
import { useAppStoreHook } from "/@/store/modules/app";
import { useUserInfo } from "/@/hooks/core/useUser";

const PageConent = defineComponent({
  name: "PageContent",
  props: pageContentProps,
  emits: [
    "createBtnClick",
    "updateBtnClick",
    "previewBtnClick",
    "statusBtnClick",
    "selectionChange"
  ],
  setup(props, { expose, emit, slots }) {
    //根据传入的api决定表格需要的操作
    const tableRef = ref<InstanceType<typeof PureTable>>(null);

    const { setSearchParams } = useAppStoreHook();

    const {
      loading,
      dataList,
      onSearch,
      getPageData,
      getBasicParams,
      getPagination,
      onBeforeAction,
      paginationConfig
    } = useRequeset(props);

    const { selects } = useSelection();
    const { action } = useActions(props.contentConfig);
    const { isSuperUser } = useUserInfo();

    const { columns, fixed, displayColumns, _raw, reset } = useColumns(
      props.contentConfig.columns
    );

    function handleSelection(value) {
      selects.value = value;
      emit("selectionChange", value);
    }

    /**
     * 渲染需要的操作按钮
     */
    function renderOperation(row) {
      const { contentConfig, powers } = props;
      const {
        apis,
        inv,
        notPreview,
        delTooltip,
        showPreview,
        showDelete,
        payProps,
        returnProps,
        isRequesetArray,
        superUserNoAction,
        superUserNoPerview
      } = contentConfig;

      const view = showPreview ? showPreview(row) : true;

      const del = showDelete ? showDelete(row) : true;

      const noAction =
        (superUserNoAction && !isSuperUser.value) || !superUserNoAction;

      const noPerview =
        (superUserNoPerview && !isSuperUser.value) || !superUserNoPerview;

      return (
        <>
          {!notPreview && view && noPerview && powers.some(i => i == "007") && (
            <Operation.Preview
              isInv={inv}
              onPreview={() => emit("previewBtnClick", row)}
            />
          )}

          {slots.custom && slots.custom(row)}

          {returnProps && noAction && (
            <Operation.Tag
              row={row}
              tagProps={returnProps}
              onReload={() => onSearch()}
            />
          )}

          {payProps && noAction && (
            <Operation.Tag
              row={row}
              tagProps={payProps}
              isPay
              onReload={() => onSearch()}
            />
          )}

          {action.update && powers.some(i => i == "005") && noAction && (
            <Operation.Update
              onUpdate={() => emit("updateBtnClick", row)}
              {...createActionProps("update", apis)}
            />
          )}

          {action.status &&
            noAction &&
            ((powers.some(i => i == "003") && row.status + "" === "0") ||
              (powers.some(i => i == "004") && row.status + "" === "1")) && (
              <Operation.Status
                row={row}
                prop={contentConfig.statusProp}
                // eslint-disable-next-line prettier/prettier
              statusRowProp={contentConfig.statusRowProp}
                onReload={() => onSearch()}
                isRequesetArray={isRequesetArray}
                {...createActionProps("status", apis)}
              />
            )}

          {del && noAction && action.delete && powers.some(i => i == "006") && (
            <Operation.Delete
              row={row}
              prop={contentConfig.deleteProp}
              delTooltip={delTooltip}
              onReload={() => onSearch()}
              {...createActionProps("delete", apis)}
            />
          )}
        </>
      );
    }

    function renderCreateButton() {
      const { contentConfig, powers } = props;
      const { superUserNoAction } = contentConfig;
      if (superUserNoAction && isSuperUser.value) {
        return null;
      }

      return (
        (slots.create && slots.create()) ||
        (action.create && powers.some(i => i == "002") && (
          <Operation.Create
            {...createActionProps("create", contentConfig.apis)}
            onCreate={() => emit("createBtnClick")}
          />
        ))
      );
    }

    function renderButtons() {
      return (
        <div style={{ display: "flex", gap: "5px" }}>
          {/* excel导出 */}

          {slots.header && slots.header()}

          {renderCreateButton()}
        </div>
      );
    }

    onBeforeRouteLeave(({ path }) => {
      //到详情页保存查询参数 到其他页面清空查询参数
      const isDetail = path.indexOf("Detail") >= 0;
      const basicParams = isDetail ? getBasicParams() : null;
      const pagination = isDetail ? getPagination() : null;
      setSearchParams(basicParams, pagination);
    });

    function renderExpand(row) {
      return slots.expand && slots.expand(row);
    }

    function renderFilterColumnns() {
      return (
        <div class="filter-columns__content">
          <div class="filter-columns_reset">
            列筛选
            <ElButton type="text" onClick={() => reset()}>
              重置
            </ElButton>
          </div>

          <ElScrollbar maxHeight={300}>
            <ElCheckboxGroup v-model={displayColumns.value}>
              {_raw.map(({ label, prop, fixed: fixedValue }, index) => {
                if (!label || label === "序号" || label === "操作") return;
                return (
                  <div class="fixed-group">
                    <ElButtonGroup>
                      <ElTooltip
                        content="固定在左边"
                        placement="top"
                        disabled={fixedValue === "left"}
                      >
                        <ElButton
                          link
                          icon={useRenderIcon("arrow-left-s-line")}
                          onClick={() => fixed("left", index)}
                        />
                      </ElTooltip>
                      <ElTooltip
                        content="固定在右边"
                        placement="top"
                        disabled={fixedValue === "right"}
                      >
                        <ElButton
                          link
                          icon={useRenderIcon("arrow-right-s-line")}
                          onClick={() => fixed("right", index)}
                        />
                      </ElTooltip>
                    </ElButtonGroup>
                    <ElCheckbox label={prop} key={prop}>
                      {label}
                    </ElCheckbox>
                  </div>
                );
              })}
            </ElCheckboxGroup>
          </ElScrollbar>
        </div>
      );
    }

    function renderPureTable(size, checkList) {
      const { contentConfig } = props;
      const { treeProps, rowKey, isTree } = contentConfig;

      return (
        <PureTable
          ref={isTree && tableRef}
          maxHeight="660px"
          border
          defaultExp
          align="left"
          showOverflowTooltip
          table-layout="auto"
          size={size}
          stripe
          rowKey={rowKey}
          columns={columns.value}
          treeProps={treeProps}
          data={dataList.value}
          checkList={checkList}
          paginationSmall={size === "small" ? true : false}
          headerCellStyle={{ background: "#fafafa", color: "#606266" }}
          onSelectionChange={handleSelection}
          defaultExpandAll
          {...paginationConfig}
          v-slots={{
            operation: ({ row }) => renderOperation(row),
            expand: ({ row }) => renderExpand(row)
          }}
        />
      );
    }

    expose({
      onSearch,
      getPageData,
      onBeforeAction
    });

    return () => {
      const { contentConfig } = props;
      const { title } = contentConfig;

      return (
        <TableProBar
          tableRef={tableRef.value?.getTableRef()}
          title={title}
          dataList={dataList.value}
          onRefresh={onSearch}
          loading={loading.value}
          v-slots={{
            buttons: () => renderButtons(),
            toolbar: () => renderFilterColumnns(),
            default: ({ size, checkList }) => renderPureTable(size, checkList)
          }}
        />
      );
    };
  }
});

export default PageConent;
